/////////////////////////////////////////////////////////////////////////////
// Name:        Menu.h
// Purpose:     The class to store a DVD Menu
// Author:      Alex Thuering
// Created:		04.11.2003
// RCS-ID:      $Id: Menu.h,v 1.27 2006/02/27 20:06:02 ntalex Exp $
// Copyright:   (c) Alex Thuering
// Licence:     GPL
/////////////////////////////////////////////////////////////////////////////

#ifndef MENU_H
#define MENU_H

#include <wx/wx.h>
#include <wx/image.h>
#include <wx/dynarray.h>

class wxXmlNode;
class wxSVGDocument;
class wxSVGElement;
class wxSVGUseElement;
class wxSVGSVGElement;

enum DVDFileType { DVDSTYLER_XML, DVDAUTHOR_XML, SPUMUX_XML };

enum VideoFormat
{
  vfPAL = 0,
  vfNTSC
};

enum MenuDrawType
{
  mdALL,
  mdBACKGROUND,
  mdBUTTONS_NORMAL,
  mdBUTTONS_HIGHLIGHTED,
  mdBUTTONS_SELECTED
};

enum MenuButtonState
{
  mbsNORMAL,
  mbsHIGHLIGHTED,
  mbsSELECTED
};

class Menu;

struct MenuObjectParam
{
  wxString name;
  wxString title;
  wxString type;
  wxString element; // id of element
  wxString attribute; // attribute name
  bool changeable; // changed if select the button
  wxColour normalColour;
  wxColour highlightedColour;
  wxColour selectedColour;
};

WX_DEFINE_ARRAY(MenuObjectParam*, MenuObjectParams);

class MenuObject
{
  public:
	MenuObject(Menu* menu, wxString fileName = wxT(""),
      int x = 0, int y = 0, wxString param = wxT(""));
	virtual ~MenuObject();
	
	inline bool IsButton() { return m_button; }
    
    inline wxString GetFileName() { return m_fileName; }
    inline wxString GetTitle() { return m_title; }
    inline wxString GetId() { return m_id; }
    
    int  GetX();
    void SetX(int value);
    int  GetY();
    void SetY(int value);
    int  GetWidth();
    void SetWidth(int value);
    int  GetHeight();
    void SetHeight(int value);
    
    inline wxRect GetBBox()
    { return wxRect(GetX(), GetY(), GetWidth(), GetHeight()); }
	
	inline bool IsDefaultSize() { return m_defaultSize; }
	inline void SetDefaultSize(bool value = true) { m_defaultSize = value; }
    void FixSize(int& width, int& height);
    void UpdateSize();
    
    int GetObjectParamsCount() { return m_params.Count(); }
    MenuObjectParam* GetObjectParam(int index) { return m_params[index]; }
    MenuObjectParam* GetObjectParam(wxString name);
    MenuObjectParam* GetInitParam();
    
    wxString GetParam(wxString name, MenuButtonState state = mbsNORMAL);
	void SetParam(wxString name, wxString value, MenuButtonState state = mbsNORMAL);
    int GetParamInt(wxString name, MenuButtonState state = mbsNORMAL);
	void SetParamInt(wxString name, int value, MenuButtonState state = mbsNORMAL);
	wxFont GetParamFont(wxString name, MenuButtonState state = mbsNORMAL);
	void SetParamFont(wxString name, wxFont value, MenuButtonState state = mbsNORMAL);
	wxColour GetParamColour(wxString name, MenuButtonState state = mbsNORMAL);
	void SetParamColour(wxString name, wxColour value, MenuButtonState state = mbsNORMAL);
    
    void ToFront();
    void Forward();
    void Backward();
    void ToBack();
    bool IsFirst();
    bool IsLast();
    
    wxString GetAction();
	void 	SetAction(wxString action);
	
	inline bool HasCustomAction() { return m_action.length()>0; }
	inline int  GetActionTsi() { return m_actionTsi; }
	inline void	SetActionTsi(int value) { m_actionTsi = value; }
	inline int  GetActionPgci() { return m_actionPgci; }
	inline void	SetActionPgci(int value) { m_actionPgci = value; }
	inline int  GetActionChapter() { return m_actionChapter; }
	inline void	SetActionChapter(int value) { m_actionChapter = value; }
	inline wxString GetCustomAction() { return m_action; }
	inline void SetCustomAction(wxString action) { m_action = action; }
	
	wxString GetDirection(int dir) { return m_direction[dir]; }
	void SetDirection(int dir, wxString value) { m_direction[dir] = value; }
	
	wxImage GetImage(int maxWidth, int maxHeight);
	
	virtual wxXmlNode* GetXML(DVDFileType type = DVDSTYLER_XML,
      bool withSVG = false);
	virtual bool PutXML(wxXmlNode* node);
	
  protected:
	bool m_button;
    wxString m_fileName;
    wxString m_title;
    wxString m_id;
    
    wxString m_action;
	int m_actionTsi;
	int m_actionPgci;
	int m_actionChapter;
	wxString m_direction[4]; // left, right, up, down button names
    
    MenuObjectParams m_params;
    wxString m_initParameter;
    
    bool m_defaultSize;
    wxString m_defaultSizeElement;
    int m_defaultSizeWidth;
    int m_defaultSizeHeight;
    
    wxString m_minSizeElement;
    int m_minSizeWidth;
    int m_minSizeHeight;
    
    wxSVGDocument* m_svg;
    wxSVGUseElement* m_use;
    wxSVGSVGElement* m_symbol;
    wxSVGSVGElement* AddSymol(wxString id, wxSVGElement* content);
    void AddUse(wxString id, int x, int y, int width, int height);
    
    bool Init(wxString filename, int x = 0, int y = 0, wxString param = wxT(""));
    bool LoadSVG(wxSVGDocument& svg, wxXmlNode* node);
    wxString GenerateId(wxString prefix);
};

WX_DEFINE_ARRAY(MenuObject*, MenuObjects);

class Menu
{
  public:
	Menu(VideoFormat videoFormat = vfPAL);
	~Menu();
    
    inline wxSVGDocument* GetSVG() { return m_svg; }
	
	static wxArrayString GetVideoFormatStrings();
	inline VideoFormat GetVideoFormat() { return m_videoFormat; }
	void SetVideoFormat(VideoFormat format);
	wxSize GetResolution();
	
	inline wxString GetStartTime() { return m_startTime; }
	inline void 	SetStartTime(wxString time) { m_startTime = time; }
	inline wxString GetEndTime() { return m_endTime; }
	inline void 	SetEndTime(wxString time) { m_endTime = time; }
	
    void SetBackground(wxString fileName);
    wxString GetBackground();
    void SetBackgroundColour(wxColour colour);
    wxColour GetBackgroundColour();
    bool HasVideoBackground();
	
	wxString AddImage(wxString fileName, int x = -1, int y = -1);
	wxString AddText(wxString text, int x = -1, int y = -1);
    wxString AddObject(wxString fileName, wxString param, int x = -1, int y = -1);
	
	int GetObjectsCount() { return m_objects.Count(); }
    MenuObject* GetObject(int index) { return m_objects[index]; }
	MenuObject* GetObject(wxString id);
    void RemoveObject(wxString id);
    void ObjectToFront(wxString id);
    void ObjectForward(wxString id);
    void ObjectBackward(wxString id);
    void ObjectToBack(wxString id);
    bool IsFirstObject(wxString id);
    bool IsLastObject(wxString id);
	
	void SetTransparentColor();
	wxImage GetImage(MenuDrawType drawType = mdALL, int width = -1, int height = -1);
	
	bool SaveSpumux(wxString fileName,  wxString btFile, wxString hlFile, wxString selFile);
	wxXmlNode* GetXML(DVDFileType type, wxXmlNode* node = NULL);
	bool PutXML(wxXmlNode* node);
    wxString AddObject(wxXmlNode* node, bool fixPosition = false);

  protected:
    wxSVGDocument* m_svg;
	VideoFormat m_videoFormat;
	wxString 	m_startTime, m_endTime;
	wxColour 	m_transpColour;
	MenuObjects m_objects;
	void FixPosition(MenuObject* obj); /** for copy & paste */
    bool IsDefElement(wxSVGElement* element);
    void RemoveChangeable(wxSVGElement* element, MenuObject* obj);
    bool RemoveNotChangeable(wxSVGElement* element, MenuObject* obj);
};

#endif // MENU_H
